import enum

from nassl.ssl_client import OpenSslVersionEnum
from sslyze.plugins.robot_plugin import RobotScanResultEnum
from sslyze.plugins.certificate_info_plugin import OcspResponseStatusEnum 

from sqlalchemy import create_engine, Column, Integer, String, DateTime, Boolean, ForeignKey, Table, Enum
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine, func
from sqlalchemy.orm import sessionmaker

from sslyze.plugins.certificate_info_plugin import CertificateInfoScanCommand
from sslyze.plugins.openssl_cipher_suites_plugin import Sslv20ScanCommand, Sslv30ScanCommand, Tlsv10ScanCommand, Tlsv11ScanCommand, Tlsv12ScanCommand, Tlsv13ScanCommand
from sslyze.plugins.compression_plugin import CompressionScanCommand
from sslyze.plugins.fallback_scsv_plugin import FallbackScsvScanCommand 
from sslyze.plugins.heartbleed_plugin import HeartbleedScanCommand 
from sslyze.plugins.openssl_ccs_injection_plugin import OpenSslCcsInjectionScanCommand 
from sslyze.plugins.session_renegotiation_plugin import SessionRenegotiationScanCommand 
from sslyze.plugins.session_resumption_plugin import SessionResumptionSupportScanCommand 
from sslyze.plugins.robot_plugin import RobotScanCommand 
from sslyze.plugins.early_data_plugin import EarlyDataScanCommand

from datetime import datetime, timedelta, timezone
from .. import logger

Basemodel = declarative_base()

def create_db(connection, verbosity = 0):
	logger.info('Creating database %s' % connection)
	engine = create_engine(connection, echo=True if verbosity > 1 else False) #'sqlite:///dump.db'	
	Basemodel.metadata.create_all(engine)
	logger.info('Done creating database %s' % connection)

def get_session(connection, verbosity = 0):
	logger.debug('Connecting to DB')
	engine = create_engine(connection, echo=True if verbosity > 1 else False) #'sqlite:///dump.db'	
	logger.debug('Creating session')
	# create a configured "Session" class
	Session = sessionmaker(bind=engine)
	# create a Session
	return Session()

class SSLYZECommand(enum.Enum):
	ALL = -1
	CERTIFICATEINFO = 0
	SSLV20 = 1
	SSLV30 = 2
	TLSV10 = 3
	TLSV11 = 4
	TLSV12 = 5
	TLSV13 = 6
	COMPRESSION = 7
	FALLBACKSCSV = 8
	HEARTBLEED = 9
	OPENSSLCCSINJECTION = 10
	SESSIONRENEGOTIATION = 11
	SESSIONRESUMPTIONSUPPORT = 12
	ROBOT = 13
	EARLYDATA = 14

command_map = {
	SSLYZECommand.ALL : [
		CertificateInfoScanCommand,
		Sslv20ScanCommand,
		Sslv30ScanCommand,
		Tlsv10ScanCommand,
		Tlsv11ScanCommand, 
		Tlsv12ScanCommand,
		Tlsv13ScanCommand,
		CompressionScanCommand,
		FallbackScsvScanCommand,
		HeartbleedScanCommand,
		OpenSslCcsInjectionScanCommand,
		SessionRenegotiationScanCommand,
		SessionResumptionSupportScanCommand,
		RobotScanCommand,
		EarlyDataScanCommand,
	],
	SSLYZECommand.CERTIFICATEINFO : [CertificateInfoScanCommand],
	SSLYZECommand.SSLV20 : [Sslv20ScanCommand],
	SSLYZECommand.SSLV30 : [Sslv30ScanCommand],
	SSLYZECommand.TLSV10 : [Tlsv10ScanCommand],
	SSLYZECommand.TLSV11 : [Tlsv11ScanCommand],
	SSLYZECommand.TLSV12 : [Tlsv12ScanCommand],
	SSLYZECommand.TLSV13 : [Tlsv13ScanCommand],
	SSLYZECommand.COMPRESSION : [CompressionScanCommand],
	SSLYZECommand.FALLBACKSCSV : [FallbackScsvScanCommand],
	SSLYZECommand.HEARTBLEED : [HeartbleedScanCommand],
	SSLYZECommand.OPENSSLCCSINJECTION : [OpenSslCcsInjectionScanCommand],
	SSLYZECommand.SESSIONRENEGOTIATION : [SessionRenegotiationScanCommand],
	SSLYZECommand.SESSIONRESUMPTIONSUPPORT : [SessionResumptionSupportScanCommand],
	SSLYZECommand.ROBOT : [RobotScanCommand],
	SSLYZECommand.EARLYDATA : [EarlyDataScanCommand],
}

command_map_rev = {
	CertificateInfoScanCommand : SSLYZECommand.CERTIFICATEINFO,
	Sslv20ScanCommand : SSLYZECommand.SSLV20,
	Sslv30ScanCommand : SSLYZECommand.SSLV30,
	Tlsv10ScanCommand : SSLYZECommand.TLSV10,
	Tlsv11ScanCommand : SSLYZECommand.TLSV11,
	Tlsv12ScanCommand : SSLYZECommand.TLSV12,
	Tlsv13ScanCommand : SSLYZECommand.TLSV13,
	CompressionScanCommand : SSLYZECommand.COMPRESSION,
	FallbackScsvScanCommand : SSLYZECommand.FALLBACKSCSV,
	HeartbleedScanCommand : SSLYZECommand.HEARTBLEED,
	OpenSslCcsInjectionScanCommand : SSLYZECommand.OPENSSLCCSINJECTION,
	SessionRenegotiationScanCommand : SSLYZECommand.SESSIONRENEGOTIATION,
	SessionResumptionSupportScanCommand : SSLYZECommand.SESSIONRESUMPTIONSUPPORT,
	RobotScanCommand : SSLYZECommand.ROBOT,
	EarlyDataScanCommand : SSLYZECommand.EARLYDATA,
}

scan_targets_table = Table('scan_targets_table', Basemodel.metadata,
    Column('scan_id', Integer, ForeignKey('scans.id')),
    Column('target_id', Integer, ForeignKey('targets.id'))
)

class Project(Basemodel):
	__tablename__ = 'projects'

	id = Column(Integer, primary_key=True)
	name = Column(String)
	created_at = Column(DateTime, default=datetime.utcnow)
	cmd = Column(String)

	def __init__(self, name, cmd):
		self.name = name
		self.cmd = cmd

class Target(Basemodel):
	__tablename__ = 'targets'

	id = Column(Integer, primary_key=True)
	scans = relationship("Scan", secondary=scan_targets_table,back_populates="targets", lazy='dynamic')
	added_at = Column(DateTime, default=datetime.utcnow)
	ip_address = Column(String)
	domain_name = Column(String)
	port = Column(Integer)
	protocol = Column(Integer)

	def get_addr(self):
		return self.domain_name if self.domain_name is not None else self.ip_address

	def get_full_addr_s(self):
		return ':'.join([self.get_addr(), str(self.port)])

	@staticmethod
	def from_line(line):
		temp = line.split(':')
		if len(temp) == 1:
			address = temp[0]
			port = 443
		else:
			address = temp[0]
			port = int(temp[1])

		try:
			ipaddress.ip_address(address)
			target = Target(ip_address = address, port = port)
		except Exception as e:
			print(e)
			target = Target(domain_name = address, port = port)
			pass		
		
		return target

	def __init__(self, ip_address = None, domain_name = None, port = None, protocol = 1):
		self.ip_address = ip_address
		self.port = port
		self.domain_name = domain_name
		self.protocol = protocol

class TargetConnectivityInfo(Basemodel):
	__tablename__ = 'targetconnectivityinfos'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	is_connectivity_ok = Column(Boolean)
	

class Scan(Basemodel):
	__tablename__ = 'scans'

	id = Column(Integer, primary_key=True)
	project_id = Column(Integer, ForeignKey('projects.id'))
	started_at = Column(DateTime, default=datetime.utcnow)


	targets = relationship("Target", secondary=scan_targets_table,back_populates="scans", lazy='dynamic')
	scan_commands = relationship("ScanCommand", back_populates="scan")

class ScanCommand(Basemodel):
	__tablename__ = 'scancommands'
	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	command = Column(Enum(SSLYZECommand))
	scan = relationship("Scan", back_populates="scan_commands")


####################################################################################
class CertificateInfoScanResult(Basemodel):
	__tablename__ = 'certificateinfoscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))
	successful_trust_store_id = Column(Integer, ForeignKey('truststores.id'))

	certificate_chain = relationship("Certificate")
	path_validation_result_list = relationship("PathValidationResult")
	path_validation_error_list = relationship("PathValidationError")
	successful_trust_store = relationship("TrustStore", foreign_keys=[successful_trust_store_id])
	verified_certificate_chain = relationship("Certificate")
	certificate_matches_hostname = Column(Boolean)
	is_leaf_certificate_ev = Column(Boolean)
	certificate_has_must_staple_extension = Column(Boolean)
	certificate_included_scts_count = Column(Integer)
	#ocsp_response = Column(Boolean)
	ocsp_response_status = Column(Enum(OcspResponseStatusEnum))
	is_ocsp_response_trusted = Column(Boolean)
	has_sha1_in_certificate_chain = Column(Boolean)
	has_anchor_in_certificate_chain = Column(Boolean)
		

class Certificate(Basemodel):
	__tablename__ = 'certificates'

	id = Column(Integer, primary_key=True)
	certificateinfoscanresult_id = Column(Integer, ForeignKey('certificateinfoscanresults.id'))
	index = Column(Integer)
	certificate_data = Column(String)

	@staticmethod
	def from_result(res_id, idx, result):
		res = Certificate()
		res.certificateinfoscanresult_id = res_id
		res.index = idx
		res.certificate_data = result
		return res

class PathValidationResult(Basemodel):
	__tablename__ = 'pathvalidationresults'

	id = Column(Integer, primary_key=True)
	certificateinfoscanresult_id = Column(Integer, ForeignKey('certificateinfoscanresults.id'))

	trust_store_id = Column(Integer, ForeignKey('truststores.id'))
	trust_store = relationship("TrustStore")
	verify_string = Column(String)
	is_certificate_trusted = Column(Boolean)

class PathValidationError(Basemodel):
	__tablename__ = 'pathvalidationerrorresults'

	id = Column(Integer, primary_key=True)
	trust_store_id = Column(Integer, ForeignKey('truststores.id'))
	certificateinfoscanresult_id = Column(Integer, ForeignKey('certificateinfoscanresults.id'))

	trust_store = relationship("TrustStore")
	error_message = Column(String)

class TrustStore(Basemodel):
	__tablename__ = 'truststores'

	id = Column(Integer, primary_key=True)
	certificateinfoscanresult_id = Column(Integer, ForeignKey('certificateinfoscanresults.id'))

	path = Column(String)
	name = Column(String)
	version = Column(String)

	@staticmethod
	def from_result(res_id, result):
		res = TrustStore()
		res.certificateinfoscanresult_id = res_id
		res.path = result.path
		res.name = result.name
		res.version = result.version
		return res



####################################################################################

class CipherSuiteScanResult(Basemodel):
	__tablename__ = 'ciphersuiterestuls'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	accepted_cipher_list = relationship("AcceptedCipherSuite", back_populates="ciphersuitescanresult")
	rejected_cipher_list = relationship("RejectedCipherSuite")
	errored_cipher_list = relationship("ErroredCipherSuite")
	preferred_cipher_id = Column(Integer, ForeignKey('scans.id'))

	@staticmethod
	def from_result(session, project_id, scan_id, target_id, result):
		pass

class AcceptedCipherSuite(Basemodel):
	__tablename__ = 'acceptedciphers'

	id = Column(Integer, primary_key=True)
	ciphersuiterestuls_id = Column(Integer, ForeignKey('ciphersuiterestuls.id'))
	ciphersuitescanresult = relationship("CipherSuiteScanResult", back_populates="accepted_cipher_list")

	name = Column(String)
	openssl_name = Column(String)
	ssl_version = Column(Enum(OpenSslVersionEnum))
	is_anonymous = Column(Boolean)
	key_size = Column(Integer)
	post_handshake_response = Column(String)

	@staticmethod
	def from_result(result):
		ac = AcceptedCipherSuite()
		ac.name = result.name
		ac.openssl_name = result.openssl_name
		ac.ssl_version = result.ssl_version
		ac.is_anonymous = result.is_anonymous
		ac.key_size = result.key_size
		ac.post_handshake_response = result.post_handshake_response
		return ac

class RejectedCipherSuite(Basemodel):
	__tablename__ = 'rejectedciphers'

	id = Column(Integer, primary_key=True)
	ciphersuiterestuls_id = Column(Integer, ForeignKey('ciphersuiterestuls.id'))

	name = Column(String)
	openssl_name = Column(String)
	ssl_version = Column(Integer)
	is_anonymous = Column(Boolean)
	handshake_error_message = Column(String)

	@staticmethod
	def from_result(result):
		rc = RejectedCipherSuite()
		rc.name = result.name
		rc.openssl_name = result.openssl_name
		rc.ssl_version = result.ssl_version
		rc.is_anonymous = result.is_anonymous
		rc.handshake_error_message = result.handshake_error_message
		return rc

class ErroredCipherSuite(Basemodel):
	__tablename__ = 'erroredciphers'

	id = Column(Integer, primary_key=True)
	ciphersuiterestuls_id = Column(Integer, ForeignKey('ciphersuiterestuls.id'))

	name = Column(String)
	openssl_name = Column(String)
	ssl_version = Column(Integer)
	is_anonymous = Column(Boolean)
	error_message = Column(String)

	@staticmethod
	def from_result(result):
		ec = ErroredCipherSuite()
		ec.name = result.name
		ec.openssl_name = result.openssl_name
		ec.ssl_version = result.ssl_version
		ec.is_anonymous = result.is_anonymous
		ec.error_message = result.error_message
		return ec

####################################################################################

class SessionResumptionSupportScanResult(Basemodel):
	__tablename__ = 'sessionresumptionscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	attempted_resumptions_nb = Column(Integer)
	successful_resumptions_nb = Column(Integer)
	failed_resumptions_nb = Column(Integer)
	errored_resumptions_list = relationship("SessionResumptionErrors")
	is_ticket_resumption_supported = Column(Boolean)
	ticket_resumption_failed_reason = Column(String)
	ticket_resumption_exception = Column(String)

class SessionResumptionErrors(Basemodel):
	__tablename__ = 'sessionresumptionerrors'

	id = Column(Integer, primary_key=True)
	resumption_id = Column(Integer, ForeignKey('sessionresumptionscanresults.id'))

	error = Column(String)

class CompressionScanResult(Basemodel):
	__tablename__ = 'compressionscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	compression_name = Column(String)
	

class FallbackScsvScanResult(Basemodel):
	__tablename__ = 'fallbackscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	supports_fallback_scsv = Column(Boolean)

class HeartbleedScanResult(Basemodel):
	__tablename__ = 'heartbleedscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	is_vulnerable_to_heartbleed = Column(Boolean)

class OpenSslCcsInjectionScanResult(Basemodel):
	__tablename__ = 'ccsinjectionscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	is_vulnerable_to_ccs_injection = Column(Boolean)

class SessionRenegotiationScanResult(Basemodel):
	__tablename__ = 'sessionrenegotiationscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	accepts_client_renegotiation = Column(Boolean)
	supports_secure_renegotiation = Column(Boolean)

class RobotScanResult(Basemodel):
	__tablename__ = 'robotscanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	robot_result_enum = Column(Enum(RobotScanResultEnum))

class EarlyDataScanResult(Basemodel):
	__tablename__ = 'earlydatascanresults'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	is_early_data_supported = Column(Boolean)

class FailedCommands(Basemodel):
	__tablename__ = 'failedcommands'

	id = Column(Integer, primary_key=True)
	scan_id = Column(Integer, ForeignKey('scans.id'))
	target_id = Column(Integer, ForeignKey('targets.id'))

	failed_command = Column(Enum(SSLYZECommand))
	error_message = Column(String)


#!/usr/bin/env python3

import logging
import csv
import os
import sys
import ipaddress

from sslyzedb.core.dbmodel import Project, Scan, Target, get_session, create_db
from sslyzedb.core.scanner import *
from sslyzedb.core.report import *
from sslyzedb.utils import *
from sslyzedb import logger

def get_connection_string(args):
	if args.sql is not None and args.sql != '':
		return args.sql
	if 'SSLYZEDB' in os.environ:
		return os.environ['SSLYZEDB']
	else:
		raise Exception('DB connection string missing! Provide if either via the "--sql" parameter or by setting the "SSLYZEDB" environment variable')

def run():
	import argparse
	parser = argparse.ArgumentParser(description='SSLZye on DB')
	parser.add_argument('-v', '--verbose', action='count', default=0)
	parser.add_argument('--sql', help='sql engine address, if not present the script will look for the "SSLYZEDB" environment variable')
	
	subparsers = parser.add_subparsers(help = 'commands')
	subparsers.required = True
	subparsers.dest = 'command'

	db_group = subparsers.add_parser('db', help='Database operations')
	db_group.add_argument('cmd', nargs='?', choices=['create'], help='Database commands.')
	db_group.add_argument('rest', nargs=argparse.REMAINDER)

	createproject_group = subparsers.add_parser('createproject', help='Creates project and gives back the project id')
	createproject_group.add_argument('name', help='Project name')

	createscan_group = subparsers.add_parser('createscan', help='Creates a new scan and adds targets')
	createscan_group.add_argument('projectid', help='Project id that defines the scope')

	addtarget_group = subparsers.add_parser('addtarget', help='Adds target(s) to scan id')
	addtarget_group.add_argument('scanid', help='Project id that defines the scope')
	addtarget_group.add_argument('source', nargs='?', choices=['file', 'stdin'], help='Database commands.')
	addtarget_group.add_argument('rest', nargs=argparse.REMAINDER)

	addcmd_group = subparsers.add_parser('addcommand', help='Adds command(s) to scan id')
	addcmd_group.add_argument('scanid', help='Project id that defines the scope')
	addcmd_group.add_argument('commands', nargs=argparse.REMAINDER, help='Scan commands')

	scan_group = subparsers.add_parser('scan', help='Start scanning')
	scan_group.add_argument('scanid', help='Scan id that defines the scope')

	report_group = subparsers.add_parser('report', help='Generate report')
	report_group.add_argument('scanid', help='Scan id that defines the scope')
	report_group.add_argument('-o', '--outfile', help='File to write the report to, otherwise STDOUT')

	args = parser.parse_args()
	
	
	###### VERBOSITY
	if args.verbose == 0:
		logging.basicConfig(level=logging.INFO)
		logger.setLevel(logging.INFO)
	elif args.verbose == 1:
		logging.basicConfig(level=logging.DEBUG)
		logger.setLevel(logging.DEBUG)
	else:
		logging.basicConfig(level=1)
		logger.setLevel(1)

	if args.command == 'db':
		if args.cmd == 'create':
			conn = get_connection_string(args)
			create_db(conn, args.verbose)

		else:
			raise Exception('Unsupported DB subcommand %s' % args.cmd)

	elif args.command == 'createproject':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		logging.debug('Creating project')
		project = Project(args.name,'')
		session.add(project)
		session.commit()
		session.refresh(project)
		session.close()
		logging.info('Created project with ID %s' % project.id)

	elif args.command == 'createscan':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		logging.debug('Creating scan')
		project = session.query(Project).get(args.projectid)
		scan = Scan()
		session.add(scan)
		session.commit()
		session.refresh(scan)
		session.close()
		logging.debug('Created scan with ID %s' % scan.id)

	elif args.command == 'addtarget':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		scan = session.query(Scan).get(args.scanid)
		if args.source == 'file':
			try:
				for filename in args.rest:
					with open(filename, 'r') as f:
						for line in f:
							line = line.strip()
							target = Target.from_line(line)
							scan.targets.append(target)

			except Exception as e:
				logging.exception('Failed to parse targets file!')
				sys.exit()
			

		else:
			for targetentry in args.rest:
				target = Target.from_line(targetentry)
				scan.targets.append(target)
		session.add(scan)
		session.commit()
		session.close()

	elif args.command == 'addcommand':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		scan = session.query(Scan).get(args.scanid)
		for command in args.commands:
			c = SSLYZECommand[command.upper()]
			sc = ScanCommand()
			sc.scan_id = args.scanid
			sc.command = c
			session.add(sc)

		session.commit()
		session.close()
		logging.debug('Commands added')

	elif args.command == 'scan':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		logging.debug('Starting scan')

		start_scanner(session, args.scanid)

	elif args.command == 'report':
		conn = get_connection_string(args)
		session = get_session(conn, args.verbose)
		logging.debug('Starting reporting')
		data_lines = generate_report(session, args.scanid)
		if args.outfile is not None:
			with open(args.outfile,'wb') as f:
				for line in data_lines:
					f.write('\t'.join(line).encode() + b'\r\n')
		else:
			for line in data_lines:
				print('\t'.join(line))

if __name__ == '__main__':
	run()


from http.server import HTTPServer,SimpleHTTPRequestHandler
from socketserver import BaseServer
import ssl

httpd = HTTPServer(('localhost', 1443), SimpleHTTPRequestHandler)
httpd.socket = ssl.wrap_socket (httpd.socket, certfile='server.pem', keyfile='server_key.pem', server_side=True)
httpd.serve_forever()



from collections import OrderedDict
from nassl.ssl_client import OpenSslVersionEnum

from sqlalchemy import and_

from .. import logger
from ..utils import *
from .dbmodel import *

class FullResult:
	HEADER = ['target','compression_supported','supports_fallback_scsv', 'is_vulnerable_to_heartbleed',  'is_vulnerable_to_ccs_injection', \
				'accepts_client_renegotiation', 'supports_secure_renegotiation', 'robot_result', 'is_early_data_supported', 'SSLV23', \
				'SSLV2','SSLV3','TLSV1', 'TLSV1_1', 'TLSV1_2', 'TLSV1_3'
				]


	def __init__(self):
		self.target = None
		self.compression_supported = None
		self.supports_fallback_scsv = None
		self.is_vulnerable_to_heartbleed = None
		self.is_vulnerable_to_ccs_injection = None
		self.accepts_client_renegotiation = None
		self.supports_secure_renegotiation = None
		self.robot_result = None
		self.is_early_data_supported = None
		self.SSLV23 = None
		self.SSLV2 = None
		self.SSLV3 = None
		self.TLSV1 = None
		self.TLSV1_1 = None
		self.TLSV1_2 = None
		self.TLSV1_3 = None

		self.accepted_ciphers = []
		self.rejected_ciphers = []
		self.errored_ciphers = []

	def from_db(session, scanid):
		scanid = int(scanid)
		results = []
		active_targets = session.query(TargetConnectivityInfo.target_id).filter(TargetConnectivityInfo.scan_id == scanid).filter(TargetConnectivityInfo.is_connectivity_ok == True)
		query = session.query(Target, CompressionScanResult.compression_name, FallbackScsvScanResult.supports_fallback_scsv, HeartbleedScanResult.is_vulnerable_to_heartbleed\
								, OpenSslCcsInjectionScanResult.is_vulnerable_to_ccs_injection, SessionRenegotiationScanResult.accepts_client_renegotiation\
								, SessionRenegotiationScanResult.supports_secure_renegotiation, RobotScanResult.robot_result_enum, EarlyDataScanResult.is_early_data_supported)\
									.filter(Target.id.in_(active_targets))\
									.join(CompressionScanResult, and_(CompressionScanResult.target_id == Target.id, CompressionScanResult.scan_id == scanid), isouter=True)\
									.join(FallbackScsvScanResult, and_(FallbackScsvScanResult.target_id == Target.id, FallbackScsvScanResult.scan_id == scanid), isouter=True)\
									.join(HeartbleedScanResult, and_(HeartbleedScanResult.target_id == Target.id, HeartbleedScanResult.scan_id == scanid), isouter=True)\
									.join(OpenSslCcsInjectionScanResult, and_(OpenSslCcsInjectionScanResult.target_id == Target.id, OpenSslCcsInjectionScanResult.scan_id == scanid), isouter=True)\
									.join(SessionRenegotiationScanResult, and_(SessionRenegotiationScanResult.target_id == Target.id, SessionRenegotiationScanResult.scan_id == scanid), isouter=True)\
									.join(RobotScanResult, and_(RobotScanResult.target_id == Target.id, RobotScanResult.scan_id == scanid), isouter=True)\
									.join(EarlyDataScanResult, and_(EarlyDataScanResult.target_id == Target.id, EarlyDataScanResult.scan_id == scanid), isouter=True)

		logger.debug('Report queriing for target properties')
		for target, compression_name, supports_fallback_scsv, is_vulnerable_to_heartbleed, is_vulnerable_to_ccs_injection, accepts_client_renegotiation, supports_secure_renegotiation,\
					robot_result_enum, is_early_data_supported in query.all():
			
			fr = FullResult()
			fr.target = target
			fr.compression_supported = compression_name
			fr.supports_fallback_scsv = supports_fallback_scsv
			fr.is_vulnerable_to_heartbleed = is_vulnerable_to_heartbleed
			fr.is_vulnerable_to_ccs_injection = is_vulnerable_to_ccs_injection
			fr.accepts_client_renegotiation = accepts_client_renegotiation
			fr.supports_secure_renegotiation = supports_secure_renegotiation
			fr.robot_result = robot_result_enum
			fr.is_early_data_supported = is_early_data_supported

			
			logger.debug('Report queriing for target supported ciphers')
			cipersuiteres = session.query(CipherSuiteScanResult.id).filter(CipherSuiteScanResult.target_id == target.id).filter(CipherSuiteScanResult.scan_id == scanid)
			for ac in session.query(AcceptedCipherSuite.ssl_version).filter(AcceptedCipherSuite.ciphersuiterestuls_id.in_(cipersuiteres)).group_by(AcceptedCipherSuite.ssl_version).all():
				fr.accepted_ciphers.append(ac[0])
			
			logger.debug('Report queriing for target rejected ciphers')
			for rc in session.query(RejectedCipherSuite.ssl_version).filter(RejectedCipherSuite.ciphersuiterestuls_id.in_(cipersuiteres)).group_by(RejectedCipherSuite.ssl_version).all():
				fr.rejected_ciphers.append(rc[0])

			logger.debug('Report queriing for target errored ciphers')
			for ec in session.query(RejectedCipherSuite.ssl_version).filter(RejectedCipherSuite.ciphersuiterestuls_id.in_(cipersuiteres)).group_by(RejectedCipherSuite.ssl_version).all():
				fr.errored_ciphers.append(ec[0])

			fr.get_cipher_status()
			results.append(fr)

		return results

	def get_cipher_status(self):
		self.SSLV23 =  self.cipher_stat_map(OpenSslVersionEnum.SSLV23)
		self.SSLV2 =   self.cipher_stat_map(OpenSslVersionEnum.SSLV2)
		self.SSLV3 =   self.cipher_stat_map(OpenSslVersionEnum.SSLV3)
		self.TLSV1 =   self.cipher_stat_map(OpenSslVersionEnum.TLSV1)
		self.TLSV1_1 = self.cipher_stat_map(OpenSslVersionEnum.TLSV1_1)
		self.TLSV1_2 = self.cipher_stat_map(OpenSslVersionEnum.TLSV1_2)
		self.TLSV1_3 = self.cipher_stat_map(OpenSslVersionEnum.TLSV1_3)

	def cipher_stat_map(self, sslversion):
		if sslversion in self.accepted_ciphers:
			return 'SUPPORTED'
		elif sslversion in self.rejected_ciphers:
			return 'REJECTED'
		elif sslversion in self.errored_ciphers:
			return 'ERRORED'
		else:
			return 'DISABLED'


	def to_dict(self):
		t = OrderedDict()
		t['target'] = self.target.get_full_addr_s()
		t['compression_supported'] = a(self.compression_supported)
		t['supports_fallback_scsv'] = a(self.supports_fallback_scsv)
		t['is_vulnerable_to_heartbleed'] = a(self.is_vulnerable_to_heartbleed)
		t['is_vulnerable_to_ccs_injection'] = a(self.is_vulnerable_to_ccs_injection)
		t['accepts_client_renegotiation'] = a(self.accepts_client_renegotiation)
		t['supports_secure_renegotiation'] = a(self.supports_secure_renegotiation)
		t['robot_result'] = a(self.robot_result)
		t['is_early_data_supported'] = a(self.is_early_data_supported)
		t['SSLV23'] = a(self.SSLV23)
		t['SSLV2'] = a(self.SSLV2)
		t['SSLV3'] = a(self.SSLV3)
		t['TLSV1'] = a(self.TLSV1)
		t['TLSV1_1'] = a(self.TLSV1_1)
		t['TLSV1_2'] = a(self.TLSV1_2)
		t['TLSV1_3'] = a(self.TLSV1_3)
		return t

	def get_hdr():
		return FullResult.HEADER

	def get_row(self, attrs):
		t = self.to_dict()
		return [str(t[x]) for x in attrs]

	def to_tsv(self, separator = '\t'):
		return separator.join(self.get_row(FullResult.get_hdr()))

def generate_report(session, scanid):
	logger.debug('generate_report')
	results = FullResult.from_db(session, scanid)
	data = []
	data.append(FullResult.get_hdr())
	for res in results:
		data.append(res.get_row(FullResult.get_hdr()))
	return data



#!/usr/bin/env python3

from cryptography.hazmat.primitives import serialization

from sslyze.ssl_settings import TlsWrappedProtocolEnum
from sslyze.concurrent_scanner import ConcurrentScanner, PluginRaisedExceptionScanResult
from sslyze.server_connectivity_tester import ServerConnectivityTester, ServerConnectivityError


from sslyze.plugins.certificate_info_plugin import CertificateInfoScanCommand
from sslyze.plugins.openssl_cipher_suites_plugin import Sslv20ScanCommand, Sslv30ScanCommand, Tlsv10ScanCommand, Tlsv11ScanCommand, Tlsv12ScanCommand, Tlsv13ScanCommand
from sslyze.plugins.compression_plugin import CompressionScanCommand
from sslyze.plugins.fallback_scsv_plugin import FallbackScsvScanCommand 
from sslyze.plugins.heartbleed_plugin import HeartbleedScanCommand 
from sslyze.plugins.openssl_ccs_injection_plugin import OpenSslCcsInjectionScanCommand 
from sslyze.plugins.session_renegotiation_plugin import SessionRenegotiationScanCommand 
from sslyze.plugins.session_resumption_plugin import SessionResumptionSupportScanCommand 
from sslyze.plugins.robot_plugin import RobotScanCommand 
from sslyze.plugins.early_data_plugin import EarlyDataScanCommand 



from .. import logger
from .dbmodel import *

def create_server_info(target):
	"""
	takes SSLServer object and creates server_info 
	"""
	logger.log(1, 'create_server_info called')
	server_tester = ServerConnectivityTester(
		hostname=target.domain_name if target.domain_name is not None else target.ip_address,
		port=target.port,
		tls_wrapped_protocol=TlsWrappedProtocolEnum(target.protocol)
	)
	logger.debug(f'\nTesting connectivity with {server_tester.hostname}:{server_tester.port}...')
	try:
		server_info = server_tester.perform()
	except Exception as e:
		logger.debug('Falied to connect to server %s:%d' % (target.domain_name if target.domain_name is not None else target.ip_address, target.port))
		raise e
	return server_info

def start_scanner(session, scan_id):
	"""
	takes database session and project id, creates a scan id and start scanning for all targets belonging to project
	"""
	logger.log(1, 'create_server_info called')
	logger.debug('enumerating targets')
	
	concurrent_scanner = ConcurrentScanner()
	scan = session.query(Scan).get(scan_id)
	for target in session.query(Target).filter(scan_id == scan_id):
		try:
			server_info = create_server_info(target)
		except Exception as e:
			logger.exception('Connecting to server failed')
			tci = TargetConnectivityInfo()
			tci.scan_id = scan.id
			tci.target_id = target.id
			tci.is_connectivity_ok = False
			session.add(tci)
			session.commit()
			continue

		server_info.target_id = target.id #importent to keep track of the id fof the target!
		tci = TargetConnectivityInfo()
		tci.scan_id = scan.id
		tci.target_id = target.id
		tci.is_connectivity_ok = True
		session.add(tci)
		session.commit()

		logger.debug('Creating scan result')

		logger.debug('Enqueueing server for a scan')
		commands = []
		for res in session.query(ScanCommand.command).filter(scan_id == scan_id).distinct(ScanCommand.command).all():
			for r in res:
				commands.append(r)

		if len(commands) == 0:
			raise Exception('Scan doesnt have any commands!')
		
		for command in commands:
			for c in command_map[command]:
				concurrent_scanner.queue_scan_command(server_info, c())


	logger.debug('Starting scanner!')

	for scan_result in concurrent_scanner.get_results():
		logger.debug(f'\nReceived result for "{scan_result.scan_command.get_title()}" '
			  f'on {scan_result.server_info.hostname}')
		logger.debug('ID: %s' % scan_result.server_info.target_id)

		if isinstance(scan_result, PluginRaisedExceptionScanResult):
			logger.info('Scan command failed: %s' % scan_result.scan_command.get_title())
			f = FailedCommands()
			f.scan_id = scan.id
			f.target_id = target.id
			f.error_message = scan_result.error_message
			f.failed_command = command_map_rev[type(scan_result.scan_command)]

			session.add(f)
			session.commit()

		elif isinstance(scan_result.scan_command, CertificateInfoScanCommand):
			res = CertificateInfoScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			

			res.certificate_matches_hostname = scan_result.certificate_matches_hostname
			res.is_leaf_certificate_ev = scan_result.is_leaf_certificate_ev
			res.certificate_has_must_staple_extension = scan_result.certificate_has_must_staple_extension
			res.certificate_included_scts_count = scan_result.certificate_included_scts_count
			#ocsp_response = Column(Boolean)
			res.ocsp_response_status = scan_result.ocsp_response_status
			res.is_ocsp_response_trusted = scan_result.is_ocsp_response_trusted
			res.has_sha1_in_certificate_chain = scan_result.has_sha1_in_certificate_chain
			res.has_anchor_in_certificate_chain = scan_result.has_anchor_in_certificate_chain

			session.add(res)
			
			session.commit()
			session.refresh(res)

			idx = 0
			for certdata in scan_result.certificate_chain:
				cert = Certificate.from_result(res.id, idx, certdata.public_bytes(serialization.Encoding.PEM))
				idx += 1
				session.add(cert)
			for valres in scan_result.path_validation_result_list:
				#pvr = PathValidationResult.from_result(res.id, valres)
				ts = TrustStore.from_result(res.id, valres.trust_store)
				pvr = PathValidationResult()
				
				pvr.certificateinfoscanresult_id = res.id
				pvr.trust_store = ts
				pvr.verify_string = valres.verify_string
				pvr.is_certificate_trusted = valres.is_certificate_trusted
				
				session.add(pvr)

			for valres in scan_result.path_validation_error_list:
				pvre = PathValidationError()
				pvre.certificateinfoscanresult_id = res.id
				ts = TrustStore.from_result(res.id, valres.trust_store)
				pvre.trust_store = ts

				session.add(pvre)

			if scan_result.successful_trust_store is not None:
				ts = TrustStore.from_result(res.id, scan_result.successful_trust_store)
				res.successful_trust_store = ts

			if scan_result.path_validation_error_list is not None:
				for valres in scan_result.path_validation_error_list:
					pvre = PathValidationError.from_result(res.id, valres)
					session.add(pvre)

			session.commit()

		elif isinstance(scan_result.scan_command, (Sslv20ScanCommand, Sslv30ScanCommand, Tlsv10ScanCommand, Tlsv11ScanCommand, Tlsv12ScanCommand, Tlsv13ScanCommand)):
			res = CipherSuiteScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			session.add(res)
			
			session.commit()
			session.refresh(res)

			for acres in scan_result.accepted_cipher_list:
				ac = AcceptedCipherSuite.from_result(acres)
				session.add(ac)
				res.accepted_cipher_list.append(ac)

			
			session.add(res)
			session.commit()


		elif isinstance(scan_result.scan_command, CompressionScanCommand):
			res = CompressionScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			#resoning for line below: documentation sais that None will be returned for compression_name
			# if no compression was set. Problem is: None is also returned in SQL when joining tables :(
			res.compression_name = scan_result.compression_name if scan_result.compression_name is not None else 'N/A'
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, FallbackScsvScanCommand):
			res = FallbackScsvScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.supports_fallback_scsv = scan_result.supports_fallback_scsv
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, HeartbleedScanCommand):
			res = HeartbleedScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.is_vulnerable_to_heartbleed = scan_result.is_vulnerable_to_heartbleed
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, OpenSslCcsInjectionScanCommand):
			res = OpenSslCcsInjectionScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.is_vulnerable_to_ccs_injection = scan_result.is_vulnerable_to_ccs_injection
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, SessionRenegotiationScanCommand):
			res = SessionRenegotiationScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.accepts_client_renegotiation = scan_result.accepts_client_renegotiation
			res.supports_secure_renegotiation = scan_result.supports_secure_renegotiation
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, SessionResumptionSupportScanCommand):
			res = SessionResumptionSupportScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.attempted_resumptions_nb = scan_result.attempted_resumptions_nb
			res.successful_resumptions_nb = scan_result.successful_resumptions_nb
			res.failed_resumptions_nb = scan_result.failed_resumptions_nb
			res.ticket_resumption_failed_reason = scan_result.ticket_resumption_failed_reason
			#res.ticket_resumption_exception = scan_result.ticket_resumption_exception
			session.add(res)
			session.commit()
			session.refresh(res)
			for errortext in scan_result.errored_resumptions_list:
				err = SessionResumptionErrors()
				err.resumption_id = res.id
				err.error = errortext
				session.add(err)
				res.errored_resumptions_list.append(err)

			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, RobotScanCommand):
			res = RobotScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.robot_result_enum = scan_result.robot_result_enum
			session.add(res)
			session.commit()

		elif isinstance(scan_result.scan_command, EarlyDataScanCommand):
			res = EarlyDataScanResult()
			res.scan_id = scan_id
			res.target_id = scan_result.server_info.target_id
			res.robot_result_enum = scan_result.is_early_data_supported
			session.add(res)
			session.commit()

	session.commit()
	session.close()
